/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://java.net/hg/bluebill-mobile~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.observation.ui;

import javax.annotation.Nonnull;
import it.tidalwave.util.thread.ThreadAssertions;
import it.tidalwave.util.thread.annotation.ThreadConfined;
import it.tidalwave.observation.Observation;
import it.tidalwave.observation.ObservationItem;
import it.tidalwave.mobile.util.DateUpdater;
import static it.tidalwave.util.thread.ThreadType.*;

/***********************************************************************************************************************
 *
 * The controller for all the activities related to the {@link ObservationUI}.
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public interface ObservationsViewController
  {
    /*******************************************************************************************************************
     *
     * Starts the sequence of operations for adding a new observation.
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=UI)
    public void startNewObservationSequence();

    /*******************************************************************************************************************
     *
     * Commits the temporary data accumulated so far into a new observation.
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=UI)
    public void commitNewObservation();

    /*******************************************************************************************************************
     *
     * Cancels any temporary data for a new observation.
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=UI)
    public void cancelNewObservation();

    /*******************************************************************************************************************
     *
     * Updates the date/time of an {@link Observation}.
     * 
     * @param  observation  the {@code Observation}  
     * @param  dateUpdater  the {@code DateUpdater}
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=UI)
    public void updateObservationDate (@Nonnull Observation observation, @Nonnull DateUpdater dateUpdater);
    
    /*******************************************************************************************************************
     *
     * Deletes all the observations stored in memory.
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=UI)
    public void deleteAllObservations();

    /*******************************************************************************************************************
     *
     * Export all the observations stored in memory into the SD card.
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=UI)
    public void exportObservations();

    /*******************************************************************************************************************
     *
     * Share all the observations stored in memory.
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=UI)
    public void shareObservations();

    /*******************************************************************************************************************
     *
     * Deletes the specific {@link ObservationItem}.
     *
     * @param  observationItem   the item to delete
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=UI)
    public void deleteObservationItem (@Nonnull ObservationItem observationItem);
  }
