/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://java.net/hg/bluebill-mobile~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.observation.ui;

import javax.annotation.Nonnull;
import it.tidalwave.observation.Observation;
import it.tidalwave.observation.ObservationItem;
import it.tidalwave.observation.ObservationSet;
import it.tidalwave.observation.ObservationVisitor;
import it.tidalwave.mobile.util.ProgressListener;
import static it.tidalwave.observation.Observation.Observation;

/***********************************************************************************************************************
 *
 * A decorator for {@link ObservationVisitor} which also notifies a {@link ProgressListener}.
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class ProgressObservationVisitorDecorator implements ObservationVisitor
  {
    private final ObservationVisitor delegate;
    private final ProgressListener progressListener;
    private int count;

    public ProgressObservationVisitorDecorator (final @Nonnull ObservationVisitor delegate,
                                                final @Nonnull ProgressListener progressListener)
      {
        this.delegate = delegate;
        this.progressListener = progressListener;
      }

    @Nonnull
    public ObservationVisitor getDelegate()
      {
        return delegate;
      }

    public void preVisit (final @Nonnull ObservationSet observationSet)
      {
        progressListener.setStepCount(observationSet.find(Observation).count());
        progressListener.setStep(count = 0);
        delegate.preVisit(observationSet);
      }

    public void postVisit (final @Nonnull Observation observation)
      {
        delegate.postVisit(observation);
        progressListener.setStep(++count);
      }
      
    // Below methods just delegate
    public void postVisit (final @Nonnull ObservationSet observationSet)
      {
        delegate.postVisit(observationSet);
      }

    public void preVisit (final @Nonnull Observation observation)
      {
        delegate.preVisit(observation);
      }

    public void visit (final @Nonnull ObservationItem observationItem)
      {
        delegate.visit(observationItem);
      }

    public void visit (final @Nonnull ObservationSet observationSet) 
      {
        delegate.visit(observationSet);
      }

    public void visit (final @Nonnull Observation observation) 
      {
        delegate.visit(observation);
      }
  }
